/*
Witch Variants
	by Lach
v2 (meme voices fix)
*/

local SKIN = "witch" // Skin to run script for

local SOUND_PREFIXES = {"hazy", "chld"} // Important!! These are based on the sound lumps provided for this character. If you change the names of the sound lumps, change the entries in these tables too!
local SOUND_SUFFIXES = {
	"wi",
	"ls",
	"h1",
	"h2",
	"a1",
	"a2",
	"b1",
	"b2",
	"sl",
	"ht",
	"gl",
}

local IDS = {
	SKSKWIN,
	SKSKLOSE,
	SKSKPAN1,
	SKSKPAN2,
	SKSKATK1,
	SKSKATK2,
	SKSKBST1,
	SKSKBST2,
	SKSKSLOW,
	SKSKHITM,
	SKSKPOWR,
}

local VARIANT_WICKED = 1
local VARIANT_HAZY = 2
local VARIANT_CHILDISH = 3

freeslot("sfx_wichwi", "sfx_wichls")

local VARIANT_SOUNDS = {
	[VARIANT_WICKED] = {
		[SKSKWIN] = sfx_wichwi,
		[SKSKLOSE] = sfx_wichls,
	},
	[VARIANT_HAZY] = {},
	[VARIANT_CHILDISH] = {},
}

for i = 1, #SOUND_PREFIXES
	local prefix = "sfx_"..SOUND_PREFIXES[i]
	for j = 1, #SOUND_SUFFIXES
		local sound = prefix..SOUND_SUFFIXES[j]
		freeslot(sound)
		VARIANT_SOUNDS[i + 1][IDS[j]] = _G[sound]
	end
end

SOUND_PREFIXES = nil // no need for these variables anymore, so let the tables be garbage collected
SOUND_SUFFIXES = nil

local skinsounds
local finishplaying = false

local function GetWitchVariant(color)
    if color <= SKINCOLOR_SKUNK
    or color >= SKINCOLOR_PIGEON and color <= SKINCOLOR_LEATHER
    or color >= SKINCOLOR_AQUA and color <= SKINCOLOR_MOONSLAM
        return VARIANT_HAZY
    end
    if color == SKINCOLOR_FAIRY
    or color >= SKINCOLOR_SALMON and color <= SKINCOLOR_BURGUNDY
    or color >= SKINCOLOR_ULTRAVIOLET and color <= SKINCOLOR_LILAC
        return VARIANT_CHILDISH
    end
    return VARIANT_WICKED
end

local function DoWitchTransform(player)
	local color = player.skincolor
	local variant = GetWitchVariant(color)
	player.witch_color = color
	player.witch_variant = variant
	if variant == VARIANT_HAZY
		player.kartspeed = 2
		player.kartweight = 2
		return
	end
	if variant == VARIANT_CHILDISH
        player.kartspeed = 5
        player.kartweight = 8
        return
	end
	player.kartspeed = skins[SKIN].kartspeed
	player.kartweight = skins[SKIN].kartweight
end

addHook("ThinkFrame", do
	if not skinsounds
		skinsounds = skins[SKIN].soundsid // grab the skin's sound IDs just once
	end
	
	for player in players.iterate
		local mo = player.mo
		if not mo
			continue
		end
		if mo.skin ~= SKIN
			if player.witch_color
				player.witch_color = 0
				player.witch_variant = 0
			end
			continue
		end
		
		if player.witch_color ~= player.skincolor
			DoWitchTransform(player)
		end
		
		for i = 1, #IDS
			local id = IDS[i]
			local sound = skinsounds[id]
			local newsound = VARIANT_SOUNDS[player.witch_variant][id]
			if newsound
				while S_SoundPlaying(mo, sound)
					S_StopSoundByID(mo, sound)
					S_StartSound(mo, newsound)
				end
			end
		end
	end
	
	local winplaying = S_IdPlaying(skinsounds[SKSKWIN])
	local loseplaying = S_IdPlaying(skinsounds[SKSKLOSE])
	
	if not finishplaying
	and (winplaying or loseplaying)
		local variant = 0
		local maxplace = 0
		for player in players.iterate // 1: find the place of the most recently finished player
			local pos = player.kartstuff[k_position]
			if player.exiting and pos > maxplace
				maxplace = pos
			end
		end
		while not (variant) // 2: find the most recently finished player playing as Witch
			for player in players.iterate
				if player.kartstuff[k_position] == maxplace
				and player.witch_variant
					variant = player.witch_variant
					break
				end
			end
			maxplace = $ - 1
		end
		variant = max($, 1)
		if winplaying
			S_StartSound(nil, VARIANT_SOUNDS[variant][SKSKWIN])
		elseif loseplaying
			S_StartSound(nil, VARIANT_SOUNDS[variant][SKSKLOSE])
		end
	end	
	
	finishplaying = winplaying or loseplaying
end)