/*
Sam Bridges Carrier Code
	Carrier Behavior by Lach, 
	Tether Behavior Inspired by fickleheart
	adapted by Bartman3010
*/

local SKIN = "sam"
local sam_weight_change = 0
local sam_disable_carrier_load = 0

local cv_carrierActive = CV_RegisterVar( --Enable Carrier
{
	name = "carrier_active",
	PossibleValue = CV_OnOff,
	defaultvalue = "On",
	flags = CV_NETVAR|CV_SHOWMODIF
}) 

local carrier_collide = CV_RegisterVar( --disables carrier collision
{
	name = "carrier_collide",
	PossibleValue = CV_OnOff,
	defaultvalue = "Off",
	flags = CV_NETVAR|CV_SHOWMODIF
})

local function samhelp(player, ...)
	CONS_Printf(player, "\x87" .. "Sam Bridges Command List")
	CONS_Printf(player, "\x87" .. "sam_help" .. "\x80" .. " displays this help text")
	CONS_Printf(player, "\x87" .. "carrier_active" ..  "\x80" .. "  to toggle carriers. Affects all players.")
	CONS_Printf(player, "\x87" .. "carrier_collide" ..  "\x80" .. " to toggle carrier collision. Affects all players.")
	CONS_Printf(player, "\x87" .. "sam_weight_set " ..  "\x80" .. " to change weight variable. 0-9 possible. Only changes your weight.")
	if not splitscreen
		CONS_Printf(player, "Splitscreen commands are available. These are availably only in splitscreen sessions.")
	end
	if splitscreen
		CONS_Printf(player, "Splitscreen players can use " .. "\x87" .. "sam2_weight_set" .. "\x80" .. ", " .. "\x87" ..  "sam3_weight_set"  .. "\x80" .. " or " .. "\x87" .. "sam4_weight_set" .. "\x80" .. " to change variables for splitscreen players.")
	end
	CONS_Printf(player, "Enter a weight of " .. "\x87" .. "0" ..  "\x80" .. " for a random weight.")
	CONS_Printf(player, "Changes made to carriers or weight will take effect on the next map.")
end

COM_AddCommand("sam_help", samhelp)

freeslot("SPR_SBCR", "SPR_SBBX", "SPR_TTOK", "MT_SB_CARR", "MT_SB_BOX", "S_SB_CARR", "S_SB_BOX", "S_SB_BOX_PAIN", "S_SB_CARR_PAIN","MT_TETHERLINK", "S_TETHERLINK")

states[S_SB_CARR] = {
	sprite = SPR_SBCR,
	frame = H
}

states[S_SB_CARR_PAIN] = {
	sprite = SPR_SBCR,
	frame = H,
	tics = 24,
	action = A_Scream
}

states[S_SB_BOX] = {
	sprite = SPR_SBBX,
	frame = A
}

states[S_SB_BOX_PAIN] = {
	sprite = SPR_SBBX,
	frame = A,
	tics = 24,
	action = A_Scream
}


mobjinfo[MT_SB_CARR] = {
	spawnstate = S_SB_CARR,
	radius = mobjinfo[MT_PLAYER].radius,
	height = mobjinfo[MT_PLAYER].height / 2,
	flags = MF_SPECIAL | MF_BOUNCE | MF_DONTENCOREMAP,
	activesound = sfx_s3k49,
	spawnhealth = 1,
	deathstate = S_SB_CARR_PAIN,
	deathsound = sfx_s3k5d
}

mobjinfo[MT_SB_BOX] = {
	spawnstate = S_SB_BOX_PAIN,
	radius = mobjinfo[MT_PLAYER].radius / 2,
	height = mobjinfo[MT_PLAYER].height / 2,
	flags = MF_SPECIAL | MF_BOUNCE | MF_DONTENCOREMAP,
	activesound = sfx_s3k49,
	spawnhealth = 1,
	deathstate = S_SB_CARR_PAIN,
	deathsound = sfx_s3k5d
}

--Tethering properties
states[S_TETHERLINK] = {sprite = SPR_TTOK}

mobjinfo[MT_TETHERLINK] = {
	spawnstate = S_TETHERLINK, 
	flags = MF_NOCLIP | MF_NOCLIPHEIGHT | MF_SCENERY | MF_NOGRAVITY,
}

//Carrier mechanics

local function killCarrierPart(mo)
	if mo.units != nil
	for i = 1, mo.units do
		local crate = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SB_BOX)
		P_InstaThrust(crate, mo.tracer.angle + P_RandomRange(-180, 180)*ANG1, FixedHypot(mo.tracer.momx, mo.tracer.momy) + 8*mo.tracer.scale) // fling them away
		if i < 6
			crate.momz = P_MobjFlip(mo.tracer)*5*mo.scale //vertical momentum 
		elseif i >= 6 and i < 12
			crate.momz = P_MobjFlip(mo.tracer)*10*mo.scale 
		elseif i >= 12 and i < 17
			crate.momz = P_MobjFlip(mo.tracer)*15*mo.scale 
		end

	end
	P_KillMobj(mo) // (Causes carriers to be destroyed, set to death state)
	end
	mo.movedir = (2*P_RandomKey(2) - 1)*ANG20
	mo.flags = $ & ~(MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOGRAVITY|MF_GRENADEBOUNCE) | MF_NOCLIPTHING // No collision
	mo.momz = P_MobjFlip(mo.tracer)*15*mo.scale // Vertical bounce at end of carrier life
	
end

local function pullPart(mo, target, length, hdist, factor) // Function pulls the carrier towards the player
	local angle = R_PointToAngle2(mo.x, mo.y, target.x, target.y)
	local zangle = R_PointToAngle2(0, mo.z, hdist, target.z)
	
	local goal = {
		["x"] = target.x - FixedMul(length, FixedMul(cos(angle), cos(zangle))),
		["y"] = target.y - FixedMul(length, FixedMul(sin(angle), cos(zangle))),
		["z"] = target.z - FixedMul(length, sin(zangle)) + (mo.eflags & MFE_VERTICALFLIP and mo.height or 0)
	}
	
	local mass = 0
	
	local vars = {"x", "y", "z"}
	
	for i = 1, #vars
		local v = vars[i]
		
		mo["mom"..v] = $ + (goal[v] - mo[v] - $)/factor
		
		/*local limit = length + abs(target["mom"..v])
		mo["mom"..v] = max(min(limit, $ + FixedMul(FRACUNIT - mass, goal[v] - mo[v])), -limit)
		if target.type == mo.type
			target["mom"..v] = max(min(limit, $ + FixedMul(mass, mo[v] - goal[v])), -limit)
		end*/
	end
end

local sam_carrier_count = 1
local tether_strand = {}
local tether_strand_count = 0

local function DrawTetherStrand(x, y, z)
	
	tether_strand_count = $+1
	
	if tether_strand[tether_strand_count] and tether_strand[tether_strand_count].valid then --Check if strand exists
		P_TeleportMove(tether_strand[tether_strand_count], x, y, z) --Move existing strand to new position, sorted
	else
		tether_strand[tether_strand_count] = P_SpawnMobj(x, y, z, MT_TETHERLINK) -- Create Tether Strand, sorted
		tether_strand[tether_strand_count].scale = (mapheaderinfo[gamemap] and mapheaderinfo[gamemap].mobj_scale or FRACUNIT)/4
	end
	
	local strand = tether_strand[tether_strand_count]
	strand.fuse = 2
	strand.color = 5

end

local function setSamWeight(player, sam_weight_change)
	--Changes Sam's weight
	if player.sam_weight_store == nil --Initialize variable
		player.sam_weight_store = 0
	end
	if player.mo.skin == SKIN -- Show you're playing as Sam
		if not sam_disable_carrier_load 
			if (sam_weight_change > 0)--Expecting a user defined variable
				player.sam_weight = sam_weight_change
				player.kartweight = player.sam_weight
				CONS_Printf(player, "\x80" .. "Sam's Weight is now " .. "\x87" .. player.sam_weight .. ".")
			elseif (sam_weight_change == 0)--Expecting a random variable, initialization
				player.sam_weight = player.sam_weight_random
				player.kartweight = player.sam_weight
			end
		end
	end
end

local function SamLoad()
	for player in players.iterate --Run through each player
		
		if not player.mo or player.spectate == true
			continue
		end

		player.sam_weight_random = P_RandomRange(1, 9) --Assigns each user a random variable for the race
		player.sam_weight = 0
		player.sam_weight_restore = 0
	
		--Map is loaded, setting Sam's weight for all players.
		
		if player.sam_weight_store == nil--if player.sam_weight == nil -- Must initialize variable
			setSamWeight(player, 0)
		elseif player.sam_weight_store == 0--elseif player.sam_weight == 0 --Assign user random variable
			setSamWeight(player, 0)
		elseif player.sam_weight_store ~= player.sam_weight -- User defined variable
		--elseif player.sam_weight_store > 0--elseif player.sam_weight > 0
			player.sam_weight_restore = player.sam_weight_store
			setSamWeight(player, player.sam_weight_store)
		end 
			/*
		elseif sam_weight_change == 0             --Randomizes for each map loaded if not user defined
			setSamWeight(player, sam_weight_change)
		elseif sam_disable_carrier_load
			player.kartweight = 5 -- Sets default weight for Sam if Carriers are off
		end */
	end
end

addHook("MapLoad", SamLoad)

addHook("MobjThinker", function(mo) 
	mo.flags = $ | MF_NOCLIPTHING | MF_NOCLIP
	if mo and mo.valid		
		if mo.tics % 2 // flash like the rocket sneakers
			mo.flags2 = $ & ~MF2_DONTDRAW
		else
			mo.flags2 = $ | MF2_DONTDRAW
		end
		
	end
end, MT_SB_BOX)

addHook("MobjThinker", function(mo) // Carrier Thinker
	if mo and mo.valid
		if not (mo.target and mo.target.valid
		and mo.tracer and mo.tracer.valid and mo.tracer.skin == SKIN
		and mo.tracer.player)
			P_RemoveMobj(mo) // Removes carriers if player is not Sam
			return
		end
		
		local player = mo.tracer.player
		
		if not mo.sb_shadow
			P_SpawnShadowMobj(mo)
			mo.sb_shadow = true
		end
		
		if mo.pmomz and not P_IsObjectOnGround(mo)
			mo.pmomz = 0
		end
		
		
		if mo.tracer.state == S_KART_PAIN and mo.health // Checks if player falls off track
			killCarrierPart(mo)
		end
		
		
		--Sets Sam's kartweight by default if carriers are turned off during game 
		if sam_disable_carrier_load and mo.health
			P_RemoveMobj(mo)--killCarrierPart(mo)
			player.kartweight = 5 -- Sets default weight for Sam if Carriers are off
		end
		if not mo.health // If carrier has no health, destroyed
			mo.angle = $ + mo.movedir // Carrier spins
			if mo.tics % 2 // Flashes object
				mo.flags2 = $ & ~MF2_DONTDRAW
			else
				mo.flags2 = $ | MF2_DONTDRAW
			end
			
			return // Ends object thinker for carriers removed from game
		end
		
		if player.powers[pw_flashing] // No collision when destroyed
			mo.flags = $ | MF_NOCLIPTHING
		else
			mo.flags = $ & ~MF_NOCLIPTHING
		end
		
		//If carrier collision is turned off, ignore all thing collision
		if carrier_collide.value == 0
			mo.flags = $ | MF_NOCLIPTHING
		end
		//mo.frame = $ | (mo.tracer.frame & ~FF_FRAMEMASK) // get FF_FULLBRIGHT and FF_TRANS flags applied
		mo.scale = mo.tracer.scale // set to player's scale
	
		mo.colorized = mo.tracer.colorized // set to player's colorization
		mo.flags2 = mo.tracer.flags2 // get MF2_DONTDRAW or MF2_OBJECTFLIP etc.
		mo.eflags = mo.tracer.eflags // get MFE_VERTICALFLIP or MFE_ splitscreen flags etc.
		
		if player.kartstuff[k_invincibilitytimer] // spawn carrier part ghosts under certain circumstances
			P_SpawnGhostMobj(mo).fuse = 4
		elseif (player.kartstuff[k_driftboost]
		or player.kartstuff[k_sneakertimer]
		or player.kartstuff[k_startboost])
			P_SpawnGhostMobj(mo).fuse = 2
		end
		
		//Here's the fun bit!
		
		local length = 6*mo.radius // max distance between carriers before they are pulled
		
		local target = mo.target
		
		local mapscale = (mapheaderinfo[gamemap] and mapheaderinfo[gamemap].mobj_scale or FRACUNIT)
		
		while target and target.valid // iterate through carriers and push away each one that is too close
			local hdist = FixedHypot(mo.x - target.x, mo.y - target.y)
			local dist = FixedHypot(hdist, abs(mo.z - target.z))
			
			if dist < length/2
				pullPart(mo, target, length/2, hdist, player.sb_factor)
			elseif target == mo.target and dist > length // if far enough away from an adjacent part, pull closer to it
				if dist > 512*mo.scale // (if stuck, become intangible until close enough)
					mo.flags = $ | MF_NOCLIP | MF_NOCLIPHEIGHT
				else
					mo.flags = $ & ~(MF_NOCLIP|MF_NOCLIPHEIGHT)
				end
				pullPart(mo, target, length, hdist, player.sb_factor)
			end
			
		//Places tether line between player, carrier(s)

		for j=5, 27 do
			local m = j*FRACUNIT/45
				--if target not player
				DrawTetherStrand(
					mo.target.x + FixedMul(mo.x - mo.target.x, m),
					mo.target.y + FixedMul(mo.y - mo.target.y, m),
					mo.target.z - 10*FRACUNIT + FixedMul((mo.z) - (mo.target.z), m) + 18*mapscale
				)
			end 
			
			target = target.target
			
		end
		
		mo.angle = R_PointToAngle2(mo.x, mo.y, mo.target.x, mo.target.y) // Face adjacent body part
				
		
	end
end, MT_SB_CARR)

// Carrier collision with players

addHook("TouchSpecial", function(part, mo)
	if carrier_collide.value
	and mo and mo.valid and mo.player // validity checks are such a bore
	and part and part.valid
	and part.tracer and part.tracer.valid and part.tracer.valid and part.tracer.player
	and mo ~= part.tracer // No collision with player the carrier is assigned to
	and not mo.player.powers[pw_flashing] // While a player is flashing, they should be intangible and immune
	and (not G_BattleGametype() or mo.player.kartstuff[k_bumper] and not mo.player.kartstuff[k_sneakertimer])
		local partsize = part.tracer.player.kartstuff[k_growshrinktimer]
		local mosize = mo.player.kartstuff[k_growshrinktimer]
		partsize = $/abs($ or 1)
		mosize = $/abs($ or 1)
		
		if mosize > partsize // running over a carrier while in growth should squish the part's owner
			K_SquishPlayer(part.tracer.player, mo, mo)
		elseif partsize > mosize // likewise, any carrier that runs over a player should squish it
			K_SquishPlayer(mo.player, part.tracer, part.tracer)
		elseif mo.player.kartstuff[k_invincibilitytimer] and not part.tracer.player.kartstuff[k_invincibilitytimer] // invinvible players hurt carrier owners
			K_SpinPlayer(part.tracer.player, mo, 0, mo)
		elseif part.tracer.player.kartstuff[k_invincibilitytimer] and not mo.player.kartstuff[k_invincibilitytimer] // invincible carriers hurt players
			K_SpinPlayer(mo.player, part.tracer, 0, part.tracer)
		else
			K_KartBouncing(part.tracer, mo) // otherwise just bump each other
		end
	end
	return true
end, MT_SB_CARR)

// Set up for body parts

addHook("ThinkFrame", do
	if leveltime == 0 // Check if the server has disabled or enabled the carriers
		if not cv_carrierActive.value
			sam_disable_carrier_load = 1
		elseif cv_carrierActive.value
			sam_disable_carrier_load = 0
		end
	end
	
	tether_strand_count = 0
	
	for player in players.iterate
		if not player.mo or player.spectate == true // Check if player object is valid, if it is valid, ensure player is not spectating.
			continue
		end

		local mo = player.mo
				
		if mo.skin ~= SKIN // if not playing as Sam, get rid of the table if necessary and then exit thinker
			player.sam_weight = 0
			if mo.sb_parts
				mo.sb_parts = nil
			end
			continue
		end
				
		if mo.state >= S_KART_SPIN and mo.state <= S_KART_PAIN // do not spawn body parts if spinning
		or G_BattleGametype() and not player.kartstuff[k_bumper]
			continue
		end
		
		if not mo.sb_parts // set up the carrier table if nonexistent
			mo.sb_parts = {[0] = mo}
		end
		
		if player.sam_weight == nil --Initialize before checking if not set.
			player.sam_weight = 5
		end
		
		--Number of carriers to appear
		if player.sam_weight >= 8
			sam_carrier_count = 2
		elseif player.sam_weight <= 7
			sam_carrier_count = 1
		end
		
		if player.sb_numparts ~= sam_carrier_count
			
			player.sb_name = player.name
			player.sb_numparts = sam_carrier_count
			player.sb_factor = player.sb_numparts > 3 and 5 or 7
		end

		--If the carrier system is active
		if not (sam_disable_carrier_load)
			for i = 1, max(player.sb_numparts, #mo.sb_parts) // check each carrier
				if i > player.sb_numparts
					if mo.sb_parts[i] and mo.sb_parts[i].valid and mo.sb_parts[i].health
						P_RemoveMobj(mo.sb_parts[i])--killCarrierPart(mo.sb_parts[i])
					end
					mo.sb_parts[i] = nil
				elseif not (mo.sb_parts[i] and mo.sb_parts[i].valid and mo.sb_parts[i].health) // if nonexistent/dead, spawn one in!
					local sign = -2*(#player % 2) + 1 // carriers curve clockwise or couterclockwise depending on player number
					local angle = mo.angle - sign*ANGLE_90 - sign*i*ANGLE_45
					
					local x = mo.x + FixedMul(2*mo.radius, cos(mo.angle + sign*ANGLE_90) + cos(angle))
					local y = mo.y + FixedMul(2*mo.radius, sin(mo.angle + sign*ANGLE_90) + sin(angle))
					mo.sb_parts[i] = P_SpawnMobj(x, y, mo.z, MT_SB_CARR)
					mo.sb_parts[i].target = mo.sb_parts[i - 1]
					mo.sb_parts[i].tracer = mo
					mo.sb_parts[i].flags2 = $ | MF2_DONTDRAW
					mo.sb_parts[i].units = 0
					
					
				
				end
				
			end
			 
			 /*if player.sam_weight_store ~= player.sam_weight and leveltime > 2 and sam_weight_set != 0
			 CONS_Printf(player, "NOTICE: Carriers turned on during race. Restoring previous weight.")
				setSamWeight(player, player.sam_weight)		 
			 end
			 */
			//Art handling
		
		--Check if player is playing as SAM
		if mo.skin == SKIN
			//If player was not assigned a value before level load, or if Sam + carrier code added during a session.
			if leveltime ~= 0 and player.sam_weight_store == nil
				CONS_Printf(player, "NOTICE: Sam skin or player node added to game mid-session. Assigning default weight.");
				player.sam_weight_store = 0;
				player.sam_weight_restore = 5;
				player.sam_weight = 5;
				CONS_Printf(player, "NOTICE: Sam's weight will be RANDOM next race.");
			end
			if player.sam_weight == 0 -- Time to assign a new variable
				//Assign user random variable, check to make sure player value wasn't null
				if player.sam_weight_restore == 0 and player.sam_weight_store ~= nil 
				CONS_Printf(player, "NOTICE: Changed to Sam mid-race. Assigning a random weight.");
					setSamWeight(player, player.sam_weight_random) -- This number was set at MapLoad. All players have a unique random number
				elseif player.sam_weight_restore ~= player.sam_weight -- User defined variable
				CONS_Printf(player, "NOTICE: Changed to Sam mid-race. Assigning a previously stored weight.");
					setSamWeight(player, player.sam_weight_restore)
				end
			end
			
		end
		
		
		--Carrier art handling
			if mo.sb_parts[1] and mo.sb_parts[1].valid
				if player.sam_weight == 1
					mo.sb_parts[1].frame = A
					mo.sb_parts[1].units = 1
				--	mo.sb_parts[2].frame = H 
				elseif player.sam_weight == 2
					mo.sb_parts[1].frame = B
					mo.sb_parts[1].units = 2
				--	mo.sb_parts[2].frame = H
				elseif player.sam_weight == 3
					mo.sb_parts[1].frame = C
					mo.sb_parts[1].units = 3
				--	mo.sb_parts[2].frame = H
				elseif player.sam_weight == 4
					mo.sb_parts[1].frame = D
					mo.sb_parts[1].units = 4
				--	mo.sb_parts[2].frame = H
				elseif player.sam_weight == 5
					mo.sb_parts[1].frame = E
					mo.sb_parts[1].units = 8
				--	mo.sb_parts[2].frame = H
				elseif player.sam_weight == 6
					mo.sb_parts[1].frame = F
					mo.sb_parts[1].units = 12
				--	mo.sb_parts[2].frame = H
				elseif player.sam_weight == 7
					mo.sb_parts[1].frame = G
					mo.sb_parts[1].units = 16
				--	mo.sb_parts[2].frame = H
				elseif player.sam_weight == 8
					mo.sb_parts[1].frame = G
					mo.sb_parts[1].units = 16
					if mo.sb_parts[2] and mo.sb_parts[2].valid
						mo.sb_parts[2].frame = E
						mo.sb_parts[2].units = 8
					end
				elseif player.sam_weight == 9
					mo.sb_parts[1].frame = G
					mo.sb_parts[1].units = 16
					if mo.sb_parts[2] and mo.sb_parts[2].valid
						mo.sb_parts[2].frame = G 
						mo.sb_parts[2].units = 16
					end
					
				end
				
			end
			
		end
		
	end
	
end)


local function enterSamWeight(player, sam_weight_change)
	if not multiplayer 
		CONS_Printf(player, "ERROR: You can only make this change in a Multiplayer game.")
		return
	end
	sam_weight_change = tonumber(sam_weight_change)
	if player.mo and player.mo.valid
	--Check if carrier system is turned off
	if sam_disable_carrier_load
		CONS_Printf(player, "ERROR: Sam's carriers are turned off.")
		return
	end
	--Check if player is a spectator
	if player.spectator
		CONS_Printf(player, "ERROR: Spectators cannot change Sam's weight.")
		return
	end
	--Check if player is someone other than Sam
	if player.mo.skin ~= SKIN
		if not splitscreen
			CONS_Printf(player, "ERROR: You must play as Sam Bridges to adjust this.")
		else
			CONS_Printf(player, "ERROR: PLAYER A must play as Sam Bridges to adjust this.")
		end
		return
	end
	--Player must hold still to make changes

	if(sam_weight_change == nil) -- Ensure player is entering a number
		CONS_Printf(player, "ERROR: You must use a number.")
		if not splitscreen
			CONS_Printf(player, "Sam's Weight is still " .. player.sam_weight .. ".")
		else
			CONS_Printf(player, "PLAYER A Sam's Weight is still " .. player.sam_weight .. ".")
		end
		return
	--Keep the variable within range
	elseif (sam_weight_change < 0)
		sam_weight_change = 0
	elseif (sam_weight_change > 9)
		sam_weight_change = 9
	end
		if sam_weight_change > 0
			player.sam_weight_store = sam_weight_change
			if not splitscreen
				CONS_Printf(player, "OK Sam! Your weight will be " .. "\x87" .. (player.sam_weight_store)  .. "\x80" .. " in the next map.")
			else
				CONS_Printf(player, "OK PLAYER A Sam! Your weight will be " .. "\x87" .. (player.sam_weight_store)  .. "\x80" .. " in the next map.")	
			end
		elseif sam_weight_change == 0
			player.sam_weight_store = sam_weight_change
			if not splitscreen
				CONS_Printf(player, "OK Sam! Your weight will be " .. "\x87" .. "RANDOM" .. "\x80" .. " in the next map.")
			else
				CONS_Printf(player, "OK PLAYER A Sam! Your weight will be " .. "\x87" .. "RANDOM" .. "\x80" .. " in the next map.")
			end
		end
		--setSamWeight(player, sam_weight_change)
	--Cannot change if player is not holding still
	/*
	elseif player.speed > 0
		CONS_Printf(player, "ERROR: You must be still in order to change your weight.")
		CONS_Printf(player, "Your weight is still " .. "\x87" .. (player.sam_weight) .. "!")
	*/end
end

local function enterSamWeight1(player, sam_weight_change)
		enterSamWeight(player, sam_weight_change)
end

local function enterSamWeight2(player, sam_weight_change)
	if not splitscreen
		CONS_Printf(player, "ERROR: No splitscreen players found.")
		CONS_Printf(player, "\x80" .. "Please use " .. "\x87" .. "sam_weight_set" ..  "\x80" .. " to change player's weight.")
		return
	end
	if splitscreen < 1
		CONS_Printf(player, "ERROR: PLAYER B is not active")
		return
	end
	if not multiplayer 
		CONS_Printf(player, "ERROR: You can only make this change in a Multiplayer game.")
		return
	end
	sam_weight_change = tonumber(sam_weight_change)
	--displayplayers[1].sam_weight_random != nil
	if displayplayers[1].mo and displayplayers[1].mo.valid
	--Check if carrier system is turned off
	if sam_disable_carrier_load
		CONS_Printf(player, "ERROR: Sam's carriers are turned off.")
		return
	end
	--Check if player is a spectator
	if displayplayers[1].spectator
		CONS_Printf(player, "ERROR: Spectators cannot change Sam's weight.")
		return
	end
	--Check if player is someone other than Sam
	if displayplayers[1].mo.skin ~= SKIN
		CONS_Printf(player, "ERROR: PLAYER B must play as Sam Bridges to adjust this.")
		return
	end
	--Player must hold still to make changes

	if(sam_weight_change == nil) -- Ensure player is entering a number
		CONS_Printf(player, "ERROR: You must use a number.")
		CONS_Printf(player, "PLAYER B Sam's Weight is still " .. displayplayers[1].sam_weight .. ".")
		return
	--Keep the variable within range
	elseif (sam_weight_change < 0)
		sam_weight_change = 0
	elseif (sam_weight_change > 9)
		sam_weight_change = 9
	end
		if sam_weight_change > 0
			displayplayers[1].sam_weight_store = sam_weight_change
			CONS_Printf(player, "OK PLAYER B Sam! Your weight will be " .. "\x87" .. (displayplayers[1].sam_weight_store)  .. "\x80" .. " in the next map.")
		elseif sam_weight_change == 0
			displayplayers[1].sam_weight_store = sam_weight_change
			CONS_Printf(player, "OK PLAYER B Sam! Your weight will be " .. "\x87" .. "RANDOM" .. "\x80" .. " in the next map.")
		end
		--setSamWeight(player, sam_weight_change)
	--Cannot change if player is not holding still
	/*
	elseif player.speed > 0
		CONS_Printf(player, "ERROR: You must be still in order to change your weight.")
		CONS_Printf(player, "Your weight is still " .. "\x87" .. (player.sam_weight) .. "!")
	*/end
end

local function enterSamWeight3(player, sam_weight_change)
	if not splitscreen
		CONS_Printf(player, "ERROR: No splitscreen players found.")
		CONS_Printf(player, "\x80" .. "Please use " .. "\x87" .. "sam_weight_set" ..  "\x80" .. " to change player's weight.")
		return
	end
	if splitscreen < 2
		CONS_Printf(player, "ERROR: PLAYER C is not active")
		return
	end
	if not multiplayer 
		CONS_Printf(player, "ERROR: You can only make this change in a Multiplayer game.")
		return
	end
	sam_weight_change = tonumber(sam_weight_change)
	--displayplayers[2].sam_weight_random != nil
	if displayplayers[2].mo and displayplayers[2].mo.valid
	--Check if carrier system is turned off
	if sam_disable_carrier_load
		CONS_Printf(player, "ERROR: Sam's carriers are turned off.")
		return
	end
	--Check if player is a spectator
	if displayplayers[2].spectator
		CONS_Printf(player, "ERROR: Spectators cannot change Sam's weight.")
		return
	end
	--Check if player is someone other than Sam
	if displayplayers[2].mo.skin ~= SKIN
		CONS_Printf(player, "ERROR: PLAYER C must play as Sam Bridges to adjust this.")
		return
	end
	--Player must hold still to make changes

	if(sam_weight_change == nil) -- Ensure player is entering a number
		CONS_Printf(player, "ERROR: You must use a number.")
		CONS_Printf(player, "PLAYER C Sam's Weight is still " .. displayplayers[2].sam_weight .. ".")
		return
	--Keep the variable within range
	elseif (sam_weight_change < 0)
		sam_weight_change = 0
	elseif (sam_weight_change > 9)
		sam_weight_change = 9
	end
		if sam_weight_change > 0
			displayplayers[2].sam_weight_store = sam_weight_change
			CONS_Printf(player, "OK PLAYER C Sam! Your weight will be " .. "\x87" .. (displayplayers[2].sam_weight_store)  .. "\x80" .. " in the next map.")
		elseif sam_weight_change == 0
			displayplayers[2].sam_weight_store = sam_weight_change
			CONS_Printf(player, "OK PLAYER C Sam! Your weight will be " .. "\x87" .. "RANDOM" .. "\x80" .. " in the next map.")
		end
		--setSamWeight(player, sam_weight_change)
	--Cannot change if player is not holding still
	/*
	elseif player.speed > 0
		CONS_Printf(player, "ERROR: You must be still in order to change your weight.")
		CONS_Printf(player, "Your weight is still " .. "\x87" .. (player.sam_weight) .. "!")
	*/end
end

local function enterSamWeight4(player, sam_weight_change)
	if not splitscreen
		CONS_Printf(player, "ERROR: No splitscreen players found.")
		CONS_Printf(player, "\x80" .. "Please use " .. "\x87" .. "sam_weight_set" ..  "\x80" .. " to change player's weight.")
		return
	end
	if splitscreen < 3
		CONS_Printf(player, "ERROR: PLAYER D is not active")
		return
	end
	if not multiplayer 
		CONS_Printf(player, "ERROR: You can only make this change in a Multiplayer game.")
		return
	end
	sam_weight_change = tonumber(sam_weight_change)
	--displayplayers[3].sam_weight_random != nil
	if displayplayers[3].mo and displayplayers[3].mo.valid
	--Check if carrier system is turned off
	if sam_disable_carrier_load
		CONS_Printf(player, "ERROR: Sam's carriers are turned off.")
		return
	end
	--Check if player is a spectator
	if displayplayers[3].spectator
		CONS_Printf(player, "ERROR: Spectators cannot change Sam's weight.")
		return
	end
	--Check if player is someone other than Sam
	if displayplayers[3].mo.skin ~= SKIN
		CONS_Printf(player, "ERROR: PLAYER D must play as Sam Bridges to adjust this.")
		return
	end
	--Player must hold still to make changes

	if(sam_weight_change == nil) -- Ensure player is entering a number
		CONS_Printf(player, "ERROR: You must use a number.")
		CONS_Printf(player, "PLAYER D Sam's Weight is still " .. displayplayers[3].sam_weight .. ".")
		return
	--Keep the variable within range
	elseif (sam_weight_change < 0)
		sam_weight_change = 0
	elseif (sam_weight_change > 9)
		sam_weight_change = 9
	end
		if sam_weight_change > 0
			displayplayers[3].sam_weight_store = sam_weight_change
			CONS_Printf(player, "OK PLAYER D Sam! Your weight will be " .. "\x87" .. (displayplayers[3].sam_weight_store)  .. "\x80" .. " in the next map.")
		elseif sam_weight_change == 0
			displayplayers[3].sam_weight_store = sam_weight_change
			CONS_Printf(player, "OK PLAYER D Sam! Your weight will be " .. "\x87" .. "RANDOM" .. "\x80" .. " in the next map.")
		end
		--setSamWeight(player, sam_weight_change)
	--Cannot change if player is not holding still
	/*
	elseif player.speed > 0
		CONS_Printf(player, "ERROR: You must be still in order to change your weight.")
		CONS_Printf(player, "Your weight is still " .. "\x87" .. (player.sam_weight) .. "!")
	*/end
end

COM_AddCommand("sam_weight_set", enterSamWeight)
COM_AddCommand("sam1_weight_set", enterSamWeight1) -- Just in case
COM_AddCommand("sam2_weight_set", enterSamWeight2)
COM_AddCommand("sam3_weight_set", enterSamWeight3)
COM_AddCommand("sam4_weight_set", enterSamWeight4)